<?php

namespace Drupal\Tests\path_alias_view_access\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Test the permission for viewing path aliases.
 *
 * This uses JSONAPI, as this is the only way in core to view a path alias.
 *
 * @group path_alias_view_access
 */
class PathAliasViewAccessKernelTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * The modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'path_alias',
    // This provides the admin UI for path aliases, whose access we check.
    'path',
    'file',
    'serialization',
    'jsonapi',
    'path_alias_view_access',
  ];

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The stacked http kernel service.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->httpKernel = $this->container->get('http_kernel');

    $this->installSchema('user', ['users_data']);
    $this->installEntitySchema('user');

    $this->installEntitySchema('path_alias');
  }

  /**
   * Tests the access permission for path alias entities.
   */
  public function testPathAliasAccess(): void {
    $path_alias = $this->entityTypeManager->getStorage('path_alias')->create([
      'path' => '/path',
      'alias' => '/alias',
      'status' => TRUE,
    ]);
    $path_alias->save();

    // A user with no access at all can't view a path_alias over JSONAPI.
    $user_no_access = $this->setUpCurrentUser();

    $request = Request::create('/jsonapi/path_alias/path_alias/' . $path_alias->uuid());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    // A user with our permission can view a path alias of JSONAPI, but not
    // access the admin UI.
    $user_with_permission = $this->setUpCurrentUser([], ['access path_aliases']);

    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create('/admin/config/search/path');
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    $request = Request::create('/admin/config/search/path/edit/' . $path_alias->id());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_FORBIDDEN, $response->getStatusCode());

    // A user with the core admin permission can do everything.
    $user_with_admin = $this->setUpCurrentUser([], ['administer url aliases']);
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create('/admin/config/search/path');
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());

    $request = Request::create('/admin/config/search/path/edit/' . $path_alias->id());
    $response = $this->httpKernel->handle($request);
    $this->assertEquals(Response::HTTP_OK, $response->getStatusCode());
  }

}
